package org.firstinspires.ftc.teamcode.runmodes;

import com.qualcomm.robotcore.hardware.Gamepad;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.pedroPathing.follower.Follower;
import org.firstinspires.ftc.teamcode.pedroPathing.localization.Pose;
import org.firstinspires.ftc.teamcode.subsystem.ClawSubsystem;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class Teleop {

    private ClawSubsystem claw;
    private ClawSubsystem.ClawState clawState;

    private Follower follower;
    private Pose startPose;

    private Telemetry telemetry;

    private Gamepad gamepad1;
    private Gamepad currentGamepad1;
    private Gamepad previousGamepad1;

    public RunAction stopDrive, startDrive;

    public double speed = 0.5;

    public Teleop(HardwareMap hardwareMap, Telemetry telemetry, Follower follower, Pose startPose, Gamepad gamepad1) {
        claw = new ClawSubsystem(hardwareMap, clawState);

        this.follower = follower;
        this.startPose = startPose;

        this.telemetry = telemetry;
        this.gamepad1 = gamepad1;

        this.currentGamepad1 = new Gamepad();
        this.previousGamepad1 = new Gamepad();

        stopDrive = new RunAction(this::stopDrive);
        startDrive = new RunAction(this::startDrive);
    }

    public void start() {
        claw.start();
        follower.setPose(startPose);
        follower.startTeleopDrive();
    }

    private void startDrive() {
        follower.startTeleopDrive();
    }

    private void stopDrive() {
        follower.breakFollowing();
    }

    public void update() {
        previousGamepad1.copy(currentGamepad1);
        currentGamepad1.copy(gamepad1);

        if (currentGamepad1.a && !previousGamepad1.a)
            claw.switchState();

        follower.setTeleOpMovementVectors(-gamepad1.left_stick_y * speed, -gamepad1.left_stick_x * speed, -gamepad1.right_stick_x * speed);
        follower.update();

        telemetry.addData("X", follower.getPose().getX());
        telemetry.addData("Y", follower.getPose().getY());
        telemetry.addData("Heading", Math.toDegrees(follower.getPose().getHeading()));

        telemetry.addData("Claw State", clawState);
        telemetry.update();
    }
}
