package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftReleaseHighChamberPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftReleaseLowChamberPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHighBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHighChamberPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHumanPlayerPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToLowBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToLowChamberPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftZeroPos;

import com.acmerobotics.dashboard.FtcDashboard;
import com.acmerobotics.dashboard.telemetry.MultipleTelemetry;
import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.HardwareMap;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class LiftSubsystem {
    private Telemetry telemetry;

    public DcMotor lift;
    private int pos, initalPos;
    public RunAction toZero, toLowBucket, toHighBucket, toLowChamber, releaseLowChamber, toHighChamber, releaseHighChamber, toHumanPlayer;

    public static int target;

    public LiftSubsystem(HardwareMap hardwareMap, Telemetry telemetry) {
        this.telemetry = telemetry;
        this.telemetry = new MultipleTelemetry(telemetry, FtcDashboard.getInstance().getTelemetry());
        lift = hardwareMap.get(DcMotor.class, "lift");
        lift.setDirection(DcMotor.Direction.REVERSE);
        lift.setMode(DcMotor.RunMode.RUN_WITHOUT_ENCODER);
        lift.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);

        toZero = new RunAction(this::toZero);
        toLowBucket = new RunAction(this::toLowBucket);
        toHighBucket = new RunAction(this::toHighBucket);
        toLowChamber = new RunAction(this::toLowChamber);
        releaseLowChamber = new RunAction(this::releaseLowChamber);
        toHighChamber = new RunAction(this::toHighChamber);
        releaseHighChamber = new RunAction(this::releaseHighChamber);
        toHumanPlayer = new RunAction(this::toHumanPlayer);
    }

    // Manual Control //
    public void manual(double n) { //(int liftPos, boolean negative) {
        lift.setPower(n);
    }

    public void setTarget(int b) {
        target = b;
    }

    public void addToTarget(int b) {
        target += b;
    }

    public void toZero() {
        setTarget(liftZeroPos);
    }

    public void toLowBucket() {
        setTarget(liftToLowBucketPos);
    }

    public void toHighBucket() {
        setTarget(liftToHighBucketPos);
    }

    public void toLowChamber() {
        setTarget(liftToLowChamberPos);
    }

    public void releaseLowChamber() {
        setTarget(liftReleaseLowChamberPos);
    }

    public void toHighChamber() {
        setTarget(liftToHighChamberPos);
    }

    public void releaseHighChamber() {
        setTarget(liftReleaseHighChamberPos);
    }

    public void toHumanPlayer() {
        setTarget(liftToHumanPlayerPos);
    }

    // Util //
    public double getPos() {
        updatePos();
        return pos;
    }

    public void updatePos() {
        pos = lift.getCurrentPosition() - initalPos;
    }

    public boolean isAtTarget() {
        return Math.abs(pos - target) < 10;
    }

    public void resetEncoder() {
        lift.setMode(DcMotor.RunMode.STOP_AND_RESET_ENCODER);
        lift.setMode(DcMotor.RunMode.RUN_WITHOUT_ENCODER);
    }

    // Init + Start //

    public void init() {
        resetEncoder();
        initalPos = lift.getCurrentPosition();
        lift.setMode(DcMotor.RunMode.RUN_WITHOUT_ENCODER);
        lift.setZeroPowerBehavior(DcMotor.ZeroPowerBehavior.BRAKE);
    }

    public void start() {
        initalPos = lift.getCurrentPosition();
        setTarget(10);
    }

}
