package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristFloor;

import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

import org.firstinspires.ftc.teamcode.util.action.Actions;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class WristSubsystem {

    public enum WristState {
        FLOOR, BUCKET
    }

    public ServoImplEx wrist;
    public WristState state;
    public RunAction floorWrist, bucketWrist;

    public WristSubsystem(HardwareMap hardwareMap, WristState wristState) {
        wrist = hardwareMap.get(ServoImplEx.class, "wrist-servo");
        wrist.resetDeviceConfigurationForOpMode();

        this.state = wristState;

        bucketWrist = new RunAction(this::bucketWrist);
        floorWrist = new RunAction(this::floorWrist);
    }

    public void setState(WristState wristState) {
        if (wristState == WristState.FLOOR) {
            wrist.setPosition(wristFloor);
            this.state = WristState.FLOOR;
        } else if (wristState == WristState.BUCKET) {
            wrist.setPosition(wristBucket);
            this.state = WristState.BUCKET;
        }
    }

    public void floorWrist() {
        setState(WristState.FLOOR);
    }

    public void bucketWrist() {
        setState(WristState.BUCKET);
    }

    public void switchState() {
        if (state == WristState.FLOOR) {
            setState(WristState.BUCKET);
        } else if (state == WristState.BUCKET) {
            setState(WristState.FLOOR);
        }
    }

    public WristState getState() {
        return this.state;
    }

    public void init() {
        Actions.runBlocking(floorWrist);
    }

    public void start() {
        Actions.runBlocking(floorWrist);
    }

    public double getPosition() {
        return this.wrist.getPosition();
    }

    public void setPosition(double position) {
        this.wrist.setPosition(position);
    }


}
